--[[
Copyright (c) 2006, Hendrik Leppkes < mail@nevcairiel.net >
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are 
met:

    * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above 
       copyright notice, this list of conditions and the following 
       disclaimer in the documentation and/or other materials provided 
       with the distribution.
    * Neither the name of the Bartender3 Development Team nor the names of 
       its contributors may be used to endorse or promote products derived 
       from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]

--[[ $Id: Bartender3_BarStates.lua 22577 2006-12-29 16:15:43Z nevcairiel $ ]]

local L = AceLibrary("AceLocale-2.2"):new("Bartender3")

-- stances, indexd by their actual name
Bartender3.DefaultStanceMap = { 
	["WARRIOR"] = {
		{
			name = L["Battle Stance"],
			short = "battle",
			defpage = 7,
		},
		{
			name = L["Defensive Stance"],
			short = "def",
			defpage = 8,
		},
		{
			name = L["Berserker Stance"],
			short = "berserker",
			defpage = 9,
		},
	},
	["DRUID"] = {
		{
			name = L["Bear Form"],
			short = "bear",
			defpage = 9,
		},
		{
			name = L["Aquatic Form"],
			short = "aqua",
			hidden = true,
		},
		{
			name = L["Cat Form"],
			short = "cat",
			defpage = 7,
		},
		{
			name = L["Travel Form"],
			short = "travel",
			hidden = true,
		},
		{
			name = L["Moonkin/Tree of Life"],
			short = "moontree",
		},
		{
			name = L["Flying Form"],
			short = "fly",
			hidden = true,
		},
		{
			name = L["Cat Form (Prowl)"],
			short = "prowl",
			defpage = 8,
			dummy = true,
		},
	},
	["ROGUE"] = {
		{
			name = L["Stealth"],
			short = "stealth",
			defpage = 7,
		},
	},
	["PRIEST"] = {
		{
			name = L["Shadowform"],
			short = "shadowform",
		},
	}
}

local function tfind(haystack, needle)
	for i,v in pairs(haystack) do
		if v == needle then return i end
	end
	return nil
end

function Bartender3:CreateStanceMap()
	local class = Bartender3.playerclass
	local stanceinfo = Bartender3.DefaultStanceMap[class]
	if not stanceinfo then return end
	
	self.stancemap = {}
	for i,v in pairs(stanceinfo) do
		if not Bartender3.db.profile.DisabledStances[v.short] then
			tinsert(self.stancemap, {
				["short"] = v.short,
				["name"] = v.name,
				["state"] = i,
				["defpage"] = v.defpage,
				["hidden"] = v.hidden,
			})
		end
	end
end

function Bartender3.Class.ActionBar.prototype:InitStates()
	if not Bartender3.stancemap then Bartender3:CreateStanceMap() end
	local class, frame = Bartender3.playerclass, self.frame
	local stanceinfo = Bartender3.stancemap
	
	-- setup states
	if self.config.States == nil then self.config.States = {} end
	
	-- Setup Stances
	if stanceinfo ~= nil then
		-- stance config is nil and we are bar1, setup default stances.
		if self.config.Stances == nil then self.config.Stances = {} end
		for _,v in pairs(Bartender3.DefaultStanceMap[class]) do 
		-- set default stance page if we are on the mainbar
			if self.mainbar and not v.hidden and self.config.Stances[v.short] == nil and v.defpage then
				self.config.Stances[v.short] = v.defpage
			end
		end
		
		local form = GetShapeshiftForm(true)
		
		if stanceinfo[form] and self.config.StatesEnabled then
			local page
			if class == "DRUID" and stanceinfo[form].short == "cat" and IsStealthed() then 
				page = self.config.Stances["prowl"]
			else
				page = self.config.Stances[stanceinfo[form].short]
			end
			frame:SetAttribute("state", page or 0)
		end
	end
	
	self:UpdateStates()
end

function Bartender3.Class.ActionBar.prototype:UpdateStates()
	if not Bartender3.stancemap then Bartender3:CreateStanceMap() end
	local frame = self.frame
	
	-- init the statebutton with our 10 bars
	for i=0,10 do
		self:AddButtonStates(i, i)
	end
	
	if self.mainbar then
		local transition = ""
		for i=0,10 do
			transition = transition .. format("%s-%s:%s;",2 * 10000 + i * 100, 2 * 10000 + i * 100 + 10, i)
		end
		frame:SetAttribute("statemap-actionbar-1", transition)
		for i=2,6 do
			frame:SetAttribute("statemap-actionbar-"..i, self:CreateManualTransitionMap(i))
		end
	end
	
	self:RefreshStatemap()
end

function Bartender3.Class.ActionBar.prototype:GetStanceState(stance)
	local state = 0
	if type(stance) == "table" then 
		if stance.hidden then 
			state = nil
		else 
			state = self.config.Stances[stance.short]
		end
	else
		state = self.config.Stances[stance]
	end
	if state == 0 or not self.config.StatesEnabled then state = nil end
	return state
end

function Bartender3.Class.ActionBar.prototype:RefreshStatemap()
	local class, frame = Bartender3.playerclass, self.frame
	local states = self.config.StatesEnabled
	
	-- setup statemaps for modifiers
	for _,v in pairs(Bartender3.statemodifiers) do
		local page = self.config.States[v]
		if page == 0 then page = nil end
		if not states or not page then
			frame:SetAttribute("statemap-"..v.."-0", nil)
			frame:SetAttribute("statemap-"..v.."-1", nil)
		else
			local trans1, trans2 = self:CreateModifierTransitionMap(page)
			frame:SetAttribute("statemap-"..v.."-0", trans2)
			frame:SetAttribute("statemap-"..v.."-1", trans1)
		end
	end
	
	-- HUGE class based block for stances -- it looks somewhat weird, try to optimize without breaking sometime
	-- optimized now - somewhat at least
	if Bartender3.stancemap ~= nil then
		frame:SetAttribute("statemap-stance-0", self.mainbar and self:CreateStanceTransitionMap(0) or "0")
		for i,v in pairs(Bartender3.stancemap) do
			local map = self:GetStanceState(v)
			if self.mainbar then
				map = self:CreateStanceTransitionMap(map)
			end
			frame:SetAttribute("statemap-stance-"..i, map)
		end
		if class == "DRUID" then
			-- stealthing options
			if self:GetStanceState("cat") and self:GetStanceState("prowl") then
				local map1, map2
				if self.mainbar then
					map1 = self:CreateStanceTransitionMap(self:GetStanceState("cat"), self:GetStanceState("prowl"))
					map2 = self:CreateStanceTransitionMap(self:GetStanceState("prowl"), self:GetStanceState("cat"))
				else
					map1 = self:GetStanceState("prowl")..":"..self:GetStanceState("cat")
					map2 = self:GetStanceState("cat")..":"..self:GetStanceState("prowl")
				end
				frame:SetAttribute("statemap-stealth-0", map1)
				frame:SetAttribute("statemap-stealth-1", map2)
			else
				frame:SetAttribute("statemap-stealth-0", nil)
				frame:SetAttribute("statemap-stealth-1", nil)
			end
		end
	end
	self:ApplyStateButton()
end

function Bartender3.Class.ActionBar.prototype:CreateModifierTransitionMap(target)
	local prefix = 1
	local manprefix = 2
	-- maps[1] is for entering the target -- save the state in the new state
	-- maps[2] is for going back to where we came -- recall state
	local maps = {[1] = "", [2] = ""}
	for i=0,10 do
		local id = prefix * 10000 + i * 100 + target
		self:AddButtonStates(id, target)
		maps[1] = maps[1] .. format("%s:%s;", i, id)
		if self.mainbar then maps[1] = maps[1] .. format("%s-%s:%s;", manprefix * 10000 + i * 100, manprefix * 10000 + i * 100 + 10, id) end
		maps[2] = maps[2] .. format("%s-%s:%s;", id - target, id - target + 10, i)
	end
	return maps[1], maps[2]
end

function Bartender3.Class.ActionBar.prototype:CreateStanceTransitionMap(stance, source)
	if not stance then return nil end
	local prefix = 2
	-- maps[1] is for entering the target -- save the state in the new state
	-- maps[2] is for going back to where we came -- recall state
	local map = ""
	if source then
		for i=2,10 do
			local id = prefix * 10000 + source * 100 + i
			local id2 = prefix * 10000 + stance * 100 + i
			--self:AddButtonStates(id, i)
			map = map .. format("%s:%s;", id, id2)
		end
		map = map .. format("%s:%s", source, stance)
	else
		for i=2,10 do
			local id2 = prefix * 10000 + stance * 100 + i
			for k=0,10 do
				local id = prefix * 10000 + k * 100 + i
				--self:AddButtonStates(id, i)
				map = map .. format("%s:%s;", id, id2)
			end
		end
		map = map .. format("*:%s", stance)
	end
	return map
end

function Bartender3.Class.ActionBar.prototype:CreateManualTransitionMap(target)
	local prefix = 2
	-- maps[1] is for entering the target -- save the state in the new state
	-- maps[2] is for going back to where we came -- recall state
	local map = ""
	for i=0,10 do
		local id = prefix * 10000 + i * 100 + target
		self:AddButtonStates(id, target)
		map = map .. format("%s:%s;", i, id)
		map = map .. format("%s-%s:%s;", id - target, id - target + 10, id)
	end
	return map
end

function Bartender3.Class.ActionBar.prototype:AddButtonStates(state, page)
	for _,v in pairs(self.buttonobjects) do
		local action = (page == 0) and v.id or (v.rid + ((page - 1) * 12))
		v:SetStateAction(state, action)
	end
	self:AddRightClickState(state)
	self:AddToStateButton(state)
end

function Bartender3.Class.ActionBar.prototype:AddToStateButton(state)
	if not self.statebutton then self.statebutton = {} end
	state = tonumber(state)
	if not tfind(self.statebutton, state) then 
		table.insert(self.statebutton, state)
	end
end

function Bartender3.Class.ActionBar.prototype:ApplyStateButton()
	local states = {}
	for _,v in pairs(self.statebutton) do
		for i=1,2 do
			states[i] = (states[i] or "") .. v .. ":S" .. v .. i .. ";"
		end
	end
	self.frame:SetAttribute("statebutton", states[1])
	self.frame:SetAttribute("statebutton2", states[2])
	--self.statebutton = nil
end
