--[[
Copyright (c) 2006, Hendrik Leppkes < mail@nevcairiel.net >
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are 
met:

    * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above 
       copyright notice, this list of conditions and the following 
       disclaimer in the documentation and/or other materials provided 
       with the distribution.
    * Neither the name of the Bartender3 Development Team nor the names of 
       its contributors may be used to endorse or promote products derived 
       from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]
--[[ $Id: Style.lua 22271 2006-12-27 13:55:25Z nevcairiel $ ]]

local BT3DLPadding = 3

function Bartender3:RefreshStyle(button, bar)
	if not button.icon then return end
	local style = bar.config.Style
	if style == "Zoomed" or style == "Dreamlayout" then
		-- zoom the texture
		button.icon:SetTexCoord(0.07,0.93,0.07,0.93)
	else -- default
		-- restore zoom
		button.icon:SetTexCoord(0,1,0,1)
	end
	-- dreamlayout
	if style == "Dreamlayout" then
		if not button.dreamlayout then
			button.dreamlayout = self:Dreamlayout_CreateBG(button)
		end
		button.icon:ClearAllPoints()
		button.icon:SetPoint("TOPLEFT", button.dreamlayout, "TOPLEFT", BT3DLPadding, -BT3DLPadding)
		button.icon:SetPoint("BOTTOMRIGHT", button.dreamlayout, "BOTTOMRIGHT",  -BT3DLPadding, BT3DLPadding)
		if button.class and not HasAction(button.class:PagedID()) then button.dreamlayout:Hide() 
		else button.dreamlayout:Show() end
	else
		-- hide the dreamlayout frame
		if button.dreamlayout then
			button.dreamlayout:Hide()
			button.icon:ClearAllPoints()
			button.icon:SetAllPoints(button)
		end
	end
end

function Bartender3:Dreamlayout_CreateBG(button)
	local frame = CreateFrame("Frame", button:GetName().."DL", button)
	frame:ClearAllPoints()
	frame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 1, edgeFile = "", edgeSize = 0, insets = {left = 0, right = 0, top = 0, bottom = 0},})
	frame:SetBackdropColor(0, 0, 0, 0.6)
	frame:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
	frame:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, -0)
	frame:SetFrameLevel(0)
	return frame
end
